package scg.ac.ni.pmf.newgraph.plugin.dpd;

import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import scg.ac.ni.pmf.newgraph.dpd.DPD;
import scg.ac.ni.pmf.newgraph.graphdata.EdgeData;
import scg.ac.ni.pmf.newgraph.graphdata.Graph;
import scg.ac.ni.pmf.newgraph.graphdata.GraphData;
import scg.ac.ni.pmf.newgraph.graphdata.Vertex;
import scg.ac.ni.pmf.newgraph.graphdata.VertexData;
import scg.ac.ni.pmf.newgraph.graphdata.WeightVertexInt;
import scg.ac.ni.pmf.newgraph.param.Parametrizable;

/**
 * Builds a generalized line graph.
 * 
 * @author <a href="mailto: brankovv@beotel.yu">Vladimir Brankov</a>
 */
public class GeneralizedLineGraph implements DPD, Parametrizable {

	public void create(Graph graph) {
		Map cp = new HashMap();
		
		// create coctail party graphs
		for(Enumeration i = source.vertices(); i.hasMoreElements(); ) {
			VertexData v = (VertexData) i.nextElement();
			
			Vector vector = new Vector();
			int a = 2 * source.getWeight(v, weight);
			double x = v.getX(), y = v.getY();
			for(int j = 0; j < a; j++) {
				Vertex u = graph.createVertex(x + CP_RADIUS * Math.cos(2 * Math.PI * j / a),
						y + CP_RADIUS * Math.sin(2 * Math.PI * j / a));
				vector.add(u);
				for(int k = 0; k < j; k++)
					if(j - k != a / 2)
						graph.createEdge((Vertex) vector.elementAt(k), u);
			}
			cp.put(v, vector);
		}
		
		Map edgesToVertices = new HashMap();
		
		// create line graph
		for(Enumeration i = source.edges(); i.hasMoreElements(); ) {
			EdgeData edge = (EdgeData) i.nextElement();
			VertexData va = edge.getVertexA(), vb = edge.getVertexB();
			
			Vertex v = graph.createVertex((va.getX() + vb.getX()) / 2,
					(va.getY() + vb.getY()) / 2);
			for(Enumeration j = source.edges(); j.hasMoreElements(); ) {
				EdgeData eb = (EdgeData) j.nextElement();
				if(edge.getIDA() == eb.getIDA() || edge.getIDA() == eb.getIDB() ||
						edge.getIDB() == eb.getIDA() || edge.getIDB() == eb.getIDB()) {
					Vertex w = (Vertex) edgesToVertices.get(eb);
					if(w != null)
						graph.createEdge(v, w);
				}
			}
			
			for(Iterator j = ((Vector) cp.get(va)).iterator(); j.hasNext(); ) {
				Vertex w = (Vertex) j.next();
				graph.createEdge(v, w);
			}
			
			for(Iterator j = ((Vector) cp.get(vb)).iterator(); j.hasNext(); ) {
				Vertex w = (Vertex) j.next();
				graph.createEdge(v, w);
			}
			
			edgesToVertices.put(edge, v);
		}
	}
	
	private static final double CP_RADIUS = 30;

	public String getName() {
		return "Generalized Line Graph";
	}

	public String getDescription() {
		return "Builds a generalized line graph";
	}

	public String checkParameters() {
		return null;
	}
	
	private WeightVertexInt weight;

	public WeightVertexInt getWeight() {
		return weight;
	}
	
	public void setWeight(WeightVertexInt weight) {
		this.weight = weight;
	}
	
	private GraphData source;
	
	public GraphData getSource() {
		return source;
	}
	
	public void setSource(GraphData source) {
		this.source = source;
	}
	
}
