package scg.ac.ni.pmf.newgraph.plugin.dpd;

import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

import scg.ac.ni.pmf.newgraph.dpd.DPD;
import scg.ac.ni.pmf.newgraph.graphdata.EdgeData;
import scg.ac.ni.pmf.newgraph.graphdata.Graph;
import scg.ac.ni.pmf.newgraph.graphdata.GraphData;
import scg.ac.ni.pmf.newgraph.graphdata.Vertex;
import scg.ac.ni.pmf.newgraph.graphdata.VertexData;
import scg.ac.ni.pmf.newgraph.param.Parametrizable;

/**
 * 
 * @author <a href="mailto: brankovv@beotel.yu">Vladimir Brankov</a>
 */
public class LineGraph implements DPD, Parametrizable {

	public String getName() {
		return "Line Graph";
	}

	public String getDescription() {
		return "Creates a line graph of the source graph";
	}

	public void create(Graph graph) {
		
		// create vertices
		Map edgesToVertices = new HashMap(), verticesToEdges = new HashMap();
		for(Enumeration i = source.edges(); i.hasMoreElements(); ) {
			EdgeData ed = (EdgeData) i.nextElement();
			VertexData va = ed.getVertexA(), vb = ed.getVertexB();
			Vertex v = graph.createVertex((va.getX() + vb.getX()) / 2, (va.getY() + vb.getY()) / 2);
			edgesToVertices.put(ed, v);
			verticesToEdges.put(v, ed);
		}
		
		for(Enumeration i = source.edges(); i.hasMoreElements(); ) {
			EdgeData ea = (EdgeData) i.nextElement();
			Vertex va = (Vertex) edgesToVertices.get(ea);
			for(Enumeration j = source.edges(); j.hasMoreElements(); ) {
				EdgeData eb = (EdgeData) j.nextElement();
				Vertex vb = (Vertex) edgesToVertices.get(eb);
				if(ea != eb && (ea.isIncident(eb.getVertexA()) || ea.isIncident(eb.getVertexB())))
					graph.createEdge(va, vb);
			}
		}
	}
	
	private GraphData source;

	public GraphData getSource() {
		return source;
	}
	public void setSource(GraphData source) {
		this.source = source;
	}

	public String checkParameters() {
		return null;
	}
}
