import java.util.Enumeration;

import scg.ac.ni.pmf.newgraph.graphdata.GraphData;
import scg.ac.ni.pmf.newgraph.graphdata.VertexData;
import scg.ac.ni.pmf.newgraph.invariants.InvariantInt;
import scg.ac.ni.pmf.newgraph.param.Parametrizable;

public class OrderKInvariant implements InvariantInt, Parametrizable {

	public String getName() {
		return "Order K";
	}

	public String getDescription() {
		return "The number of vertices of the degree K.";
	}
	
	public int calculate(GraphData data) {
		int n = 0;
		for(Enumeration i = data.vertices(); i.hasMoreElements(); ) {
			VertexData vertex = (VertexData) i.nextElement();
			if(compare == COMPARE_Less_Or_Equals) {
				if(vertex.degree() <= k)
					n++;
			} else if(compare == COMPARE_Equals) {
				if(vertex.degree() == k)
					n++;
			} else if(compare == COMPARE_Greater_Or_Equals) {
				if(vertex.degree() >= k)
					n++;
			}
		}
		return n;
	}

	private int k;

	public int getK() {
		return k;
	}

	public void setK(int k) {
		this.k = k;
	}

	public static final byte COMPARE_Less_Or_Equals = 1;
	public static final byte COMPARE_Equals = 2;
	public static final byte COMPARE_Greater_Or_Equals = 3;

	private byte compare;

	public byte getCompare() {
		return compare;
	}

	public void setCompare(byte compare) {
		this.compare = compare;
	}

	public String getCompareDesc() {
		return "Vertex degree compared to K";
	}

	public String checkParameters() {
		if(k <= 0)
			return "K must be positive";
		else
			return null;
	}

}
