import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

import cern.colt.matrix.*;
import cern.colt.matrix.impl.*;
import cern.colt.matrix.linalg.*;

import scg.ac.ni.pmf.newgraph.graphdata.Graph;
import scg.ac.ni.pmf.newgraph.graphdata.GraphData;
import scg.ac.ni.pmf.newgraph.graphdata.Vertex;
import scg.ac.ni.pmf.newgraph.graphdata.VertexData;
import scg.ac.ni.pmf.newgraph.invariants.InvariantDoubleArray;

public class CheckMLTD implements InvariantDoubleArray
{
        public String getName()
        {
            return "Check MLTD";
        }
        
        public String getDescription()
        {
            return "Check the majorization between " +
                   "transposed degrees and Laplacian spectrum";
        }
        
        public double[] calculate(GraphData data)
        {
            int n = data.getOrder();
            
            // calculate transposed degrees
            int[] transposed = transposedDegrees(data);

            // calculate Laplacian spectrum
            double[] lapeig = laplacianSpectrum(data);

            // form the differences between the corresponding sums
            // note that Laplacian eigenvalues are sorted in reverse order!!!
            double[] diff = new double[n];
            diff[0] = transposed[0] - lapeig[n-1];
            for (int i=1; i<n; i++)
                diff[i] = diff[i-1] + transposed[i] - lapeig[n-i-1];
            
            return diff;
        }                
        
        private int[] transposedDegrees(GraphData data)
        {
            int n = data.getOrder();
            int[] transposed = new int[n];
 
            for (int i=0; i<n; i++)
                transposed[i] = 0;

            for (Enumeration i = data.vertices(); i.hasMoreElements(); ) {
                VertexData u = (VertexData) i.nextElement();
                int d = data.degree(u.getID());
                for (int j=0; j<d; j++)
                    transposed[j]++;
            }
            
            return transposed;
        }
        
        private double[] laplacianSpectrum(GraphData data)
        {
                int n = data.getOrder();
                int[][] lapMatrix = data.laplacianMatrix();
                double[][] dlm = new double[n][n];
                
                // from int to double matrix
                for (int i=0; i<n; i++)
                    for (int j=0; j<n; j++)
                        dlm[i][j] = lapMatrix[i][j];
         
                // find a generalized spectrum
                DoubleMatrix2D cernAdj = new DenseDoubleMatrix2D(dlm);
                EigenvalueDecomposition cernDecomposition = new EigenvalueDecomposition(cernAdj);
                DoubleMatrix1D genspectrum = cernDecomposition.getRealEigenvalues();
                return genspectrum.toArray();
        }
}
