import cern.colt.matrix.*;
import cern.colt.matrix.impl.*;
import cern.colt.matrix.linalg.*;

import scg.ac.ni.pmf.newgraph.graphdata.GraphData;
import scg.ac.ni.pmf.newgraph.graphdata.VertexData;
import scg.ac.ni.pmf.newgraph.invariants.InvariantDoubleArray;
// import scg.ac.ni.pmf.newgraph.param.Parametrizable;

public class NormalizedSpectrum implements InvariantDoubleArray {

        public String getName() {
                return "Spectrum, normalized";
        }

        public String getDescription() {
                return "Each eigenvalue is divided by the largest eigenvalue.";
        }
        
        public double[] calculate(GraphData data) {
                int n = data.getOrder();
                int[][] adj = data.adjacencyMatrix();
                double genadj[][] = new double[n][n];
 
                // form a double adjacency matrix
                for (int i=0; i<n; i++)
                  for (int j=0; j<n; j++)
                     genadj[i][j]=adj[i][j];
        
                // find a spectrum
                DoubleMatrix2D cernAdj = new DenseDoubleMatrix2D(genadj);
                EigenvalueDecomposition cernDecomposition = new EigenvalueDecomposition(cernAdj);
                DoubleMatrix1D genspectrum = cernDecomposition.getRealEigenvalues();

                // normalize it
                double index = genspectrum.get(n-1);
                for (int i=0; i<n-1; i++)
                    genspectrum.set(i, genspectrum.get(i)/index);
                genspectrum.set(n-1, 1.0);
                return genspectrum.toArray();
        }
}
